import { App, Modal, ToggleComponent } from "obsidian";

export class ReadingProgressModal extends Modal {
	private value: number;
	private percentage: boolean = false;

	constructor(
		app: App,
		private readonly onSubmit: (pageNumber: number) => void
	) {
		super(app);
		this.value = 0;
	}

	onOpen(): void {
		const { contentEl } = this;
		contentEl.classList.add("obt-reading-progress");
		contentEl.createEl("h2", { text: "Enter Reading Progress" });

		contentEl.createDiv({ cls: "obt-reading-progress__desc" }, (descEl) => {
			descEl.createEl("p", {
				text: "Enter the page number or percentage of the book you have read.",
			});
			descEl.createEl("p", {
				text: "You can toggle between page number and percentage input.",
			});
		});

		const inputDiv = contentEl.createDiv({
			cls: "obt-reading-progress__input",
		});

		const inputEl = inputDiv.createEl("input", {
			type: "number",
			placeholder: "Page Number",
		});

		inputEl.addEventListener("change", (ev) => {
			this.value = Math.max(1, parseInt(inputEl.value, 10));
			(ev.target as HTMLInputElement).value = this.value.toString();
		});

		inputEl.addEventListener("keydown", (event) => {
			if (event.key === "Enter") {
				event.preventDefault();
				this.onSubmit(this.value);
				this.close();
			}
		});

		contentEl.createDiv({ cls: "obt-reading-progress__pct" }, (pctDiv) => {
			pctDiv.createEl("label", { text: "Percentage" });
			pctDiv.createDiv(
				{ cls: "obt-reading-progress__toggle" },
				(toggleDiv) => {
					new ToggleComponent(toggleDiv)
						.setValue(this.percentage)
						.onChange((value) => {
							this.percentage = value;
							if (value) {
								inputEl.setAttribute(
									"placeholder",
									"Percentage (%)"
								);
								inputEl.setAttribute("max", "100");
							} else {
								inputEl.setAttribute(
									"placeholder",
									"Page Number"
								);
								inputEl.removeAttribute("max");
							}
						});
				}
			);
		});
	}

	onClose(): void {
		const { contentEl } = this;
		contentEl.empty();
	}

	static createAndOpen(app: App): Promise<number> {
		return new Promise((resolve) => {
			const modal = new ReadingProgressModal(app, (pageNumber) => {
				resolve(pageNumber);
			});
			modal.open();
		});
	}
}
